@@ -48,12 +48,15 @@ class DefaultsConfig(cp.ConfigParser):
 
     _lock = RLock()
 
-    def __init__(self, path: str, name: str, suffix: str) -> None:
+    def __init__(self, path: str) -> None:
         super(DefaultsConfig, self).__init__(interpolation=None)
 
-        self._path = path
-        self._name = name
-        self._suffix = suffix
+        dirname, basename = osp.split(path)
+        filename, ext = osp.splitext(basename)
+
+        self._path = dirname
+        self._name = filename
+        self._suffix = ext
 
         if not osp.isdir(osp.dirname(self._path)):
             os.makedirs(osp.dirname(self._path))
@@ -147,16 +150,14 @@ class UserConfig(DefaultsConfig):
     def __init__(
         self,
         path: str,
-        name: str,
         defaults: InputDefaultsType = None,
         load: bool = True,
         version: str = "0.0.0",
         backup: bool = False,
         remove_obsolete: bool = False,
-        suffix: str = ".ini",
     ) -> None:
         """UserConfig class, based on ConfigParser."""
-        super(UserConfig, self).__init__(path=path, name=name, suffix=suffix)
+        super(UserConfig, self).__init__(path=path)
 
         self._load = load
         self._version = self._check_version(version)
@@ -315,8 +316,8 @@ class UserConfig(DefaultsConfig):
 
     def _save_new_defaults(self, defaults: DefaultsType) -> None:
         """Save new defaults."""
-        path, name = self.get_defaults_path_name_from_version()
-        new_defaults = DefaultsConfig(path=path, name=name, suffix=self._suffix)
+        path = self.get_defaults_fpath_from_version()
+        new_defaults = DefaultsConfig(path=path)
         if not osp.isfile(new_defaults.get_config_fpath()):
             new_defaults.set_defaults(defaults)
             new_defaults.save()
@@ -492,15 +493,8 @@ class UserConfig(DefaultsConfig):
 
         if isinstance(default_value, str):
             value = raw_value
-        elif isinstance(default_value, bool):
-            value = ast.literal_eval(raw_value)
-        elif isinstance(default_value, float):
-            value = float(raw_value)
-        elif isinstance(default_value, int):
-            value = int(raw_value)
         else:
             try:
-                # Lists, tuples, None, ...
                 value = ast.literal_eval(raw_value)
             except (SyntaxError, ValueError):
                 value = raw_value
@@ -540,18 +534,15 @@ class UserConfig(DefaultsConfig):
             default_value = value
             self.set_default(section, option, default_value)
 
-        if isinstance(default_value, bool):
-            value = bool(value)
-        elif isinstance(default_value, float):
+        if isinstance(default_value, float) and isinstance(value, int):
             value = float(value)
-        elif isinstance(default_value, int):
-            value = int(value)
-        # elif isinstance(default_value, list):
-        #     value = list(value)
-        # elif isinstance(default_value, tuple):
-        #     value = tuple(value)
-        elif not isinstance(default_value, str):
-            value = repr(value)
+
+        if type(default_value) is not type(value):
+            raise ValueError(
+                f"Inconsistent config type for [{section}][{option}]. "
+                f"Expected {default_value.__class__.__name__} but "
+                f"got {value.__class__.__name__}."
+            )
 
         self._set(section, option, value)
         if save:
