@@ -5,7 +5,6 @@
 import sys
 import os
 import os.path as osp
-import platform
 import shutil
 import time
 import warnings
@@ -13,14 +12,28 @@ import logging.handlers
 import asyncio
 import random
 from concurrent.futures import ThreadPoolExecutor
-from typing import Union, List, Iterator, Dict, Set, Awaitable, Optional, Any
+from typing import (
+    Union,
+    List,
+    Iterator,
+    Dict,
+    Set,
+    Tuple,
+    Awaitable,
+    Optional,
+    Any,
+)
+import tempfile
+import mimetypes
+import difflib
 
 # external imports
 import requests
 from watchdog.events import DirDeletedEvent, FileDeletedEvent  # type: ignore
-import bugsnag  # type: ignore
-from bugsnag.handlers import BugsnagHandler  # type: ignore
 from packaging.version import Version
+from datetime import datetime, timezone
+from dropbox.files import FileMetadata
+from dropbox.sharing import RequestedVisibility
 
 try:
     from systemd import journal  # type: ignore
@@ -38,9 +51,9 @@ from .errors import (
     NotFoundError,
     BusyError,
     KeyringAccessError,
+    UnsupportedFileTypeForDiff,
 )
 from .config import MaestralConfig, MaestralState, validate_config_name
-from .notify import MaestralDesktopNotificationHandler
 from .logging import CachedHandler, SdNotificationHandler, safe_journal_sender
 from .utils import get_newer_version
 from .utils.path import (
@@ -57,39 +70,16 @@ from .utils.serializer import (
     ErrorType,
 )
 from .utils.appdirs import get_log_path, get_cache_path, get_data_path
-from .constants import BUGSNAG_API_KEY, IDLE, FileStatus, GITHUB_RELEASES_API
+from .utils.integration import get_ac_state, ACState
+from .constants import IDLE, PAUSED, CONNECTING, FileStatus, GITHUB_RELEASES_API
 
 
-__all__ = [
-    "Maestral",
-]
+__all__ = ["Maestral"]
 
 
 logger = logging.getLogger(__name__)
 
 
-# set up error reporting but do not activate
-
-bugsnag.configure(
-    api_key=BUGSNAG_API_KEY,
-    app_version=__version__,
-    auto_notify=False,
-    auto_capture_sessions=False,
-)
-
-
-def bugsnag_global_callback(notification):
-    notification.add_tab(
-        "system", {"platform": platform.platform(), "python": platform.python_version()}
-    )
-    cause = notification.exception.__cause__
-    if cause:
-        notification.add_tab("original exception", error_to_dict(cause))
-
-
-bugsnag.before_notify(bugsnag_global_callback)
-
-
 # ======================================================================================
 # Main API
 # ======================================================================================
@@ -141,19 +131,14 @@ class Maestral:
         self._conf = MaestralConfig(self._config_name)
         self._state = MaestralState(self._config_name)
 
-        # enable / disable automatic reporting of errors
-        bugsnag.configure(auto_notify=self.analytics)
-
         # set up logging
         self._log_to_stdout = log_to_stdout
         self._setup_logging()
 
         # set up sync infrastructure
-        self.client = DropboxClient(
-            config_name=self.config_name
-        )  # interface to Dbx SDK
-        self.monitor = SyncMonitor(self.client)  # coordinates sync threads
-        self.sync = self.monitor.sync  # provides core sync functionality
+        self.client = DropboxClient(config_name=self.config_name)
+        self.monitor = SyncMonitor(self.client)
+        self.sync = self.monitor.sync
 
         self._check_and_run_post_update_scripts()
 
@@ -241,8 +226,7 @@ class Maestral:
     def _setup_logging(self) -> None:
         """
         Sets up logging to log files, status and error properties, desktop notifications,
-        the systemd journal if available, bugsnag if error reports are enabled, and to
-        stdout if requested.
+        the systemd journal if available, and to stdout if requested.
         """
 
         self._logger = logging.getLogger("maestral")
@@ -308,16 +292,6 @@ class Maestral:
         self._log_handler_error_cache.setLevel(logging.ERROR)
         self._logger.addHandler(self._log_handler_error_cache)
 
-        # log errors to desktop notifications
-        self._log_handler_desktop_notifier = MaestralDesktopNotificationHandler()
-        self._log_handler_desktop_notifier.setLevel(logging.WARNING)
-        self._logger.addHandler(self._log_handler_desktop_notifier)
-
-        # log to bugsnag (disabled by default)
-        self._log_handler_bugsnag = BugsnagHandler()
-        self._log_handler_bugsnag.setLevel(logging.ERROR if self.analytics else 100)
-        self._logger.addHandler(self._log_handler_bugsnag)
-
     # ==== methods to access config and saved state ====================================
 
     @property
@@ -471,20 +445,6 @@ class Maestral:
         level = self.log_level if enabled else 100
         self.log_handler_stream.setLevel(level)
 
-    @property
-    def analytics(self) -> bool:
-        """Enables or disables logging of errors to bugsnag."""
-        return self._conf.get("app", "analytics")
-
-    @analytics.setter
-    def analytics(self, enabled: bool) -> None:
-        """Setter: analytics."""
-
-        bugsnag.configure(auto_notify=self.analytics)
-        self._log_handler_bugsnag.setLevel(logging.ERROR if enabled else 100)
-
-        self._conf.set("app", "analytics", enabled)
-
     @property
     def notification_snooze(self) -> float:
         """Snooze time for desktop notifications in minutes. Defaults to 0.0 if
@@ -538,21 +498,11 @@ class Maestral:
         """Indicates if the initial download has already occurred (read only)."""
         return self.sync.local_cursor == 0 or self.sync.remote_cursor == ""
 
-    @property
-    def syncing(self) -> bool:
-        """Indicates if Maestral is syncing (read only). It will be ``True`` if syncing
-        is not paused by the user *and* Maestral is connected to the internet."""
-        return (
-            self.monitor.syncing.is_set()
-            or self.monitor.startup.is_set()
-            or self.sync.busy()
-        )
-
     @property
     def paused(self) -> bool:
         """Indicates if syncing is paused by the user (read only). This is set by
         calling :meth:`pause`."""
-        return self.monitor.paused_by_user.is_set() and not self.sync.busy()
+        return not self.monitor.autostart.is_set() and not self.sync.busy()
 
     @property
     def running(self) -> bool:
@@ -568,13 +518,18 @@ class Maestral:
         if self.pending_link:
             return False
         else:
-            return self.monitor.connected.is_set()
+            return self.monitor.connected
 
     @property
     def status(self) -> str:
         """The last status message (read only). This can be displayed as information to
         the user but should not be relied on otherwise."""
-        return self._log_handler_info_cache.getLastMessage()
+        if self.paused:
+            return PAUSED
+        elif not self.connected:
+            return CONNECTING
+        else:
+            return self._log_handler_info_cache.getLastMessage()
 
     @property
     def sync_errors(self) -> List[ErrorType]:
@@ -641,7 +596,7 @@ class Maestral:
             'up to date', 'error', or 'unwatched' (for files outside of the Dropbox
             directory). This will always be 'unwatched' if syncing is paused.
         """
-        if not self.syncing:
+        if not self.running:
             return FileStatus.Unwatched.value
 
         local_path = osp.realpath(local_path)
@@ -865,6 +820,101 @@ class Maestral:
 
         return entries
 
+    def get_file_diff(self, old_rev: str, new_rev: Optional[str] = None) -> List[str]:
+        """
+        Compare to revisions of a text file using Python's difflib. The versions will be
+        downloaded to temporary files. If new_rev is None, the old revision will be
+        compared to the corresponding local file, if any.
+
+        :param old_rev: Identifier of old revision.
+        :param new_rev: Identifier of new revision.
+        :returns: Diff as a list of strings (lines).
+        :raises UnsupportedFileTypeForDiff: if file type is not supported.
+        :raises UnsupportedFileTypeForDiff: if file content could not be decoded.
+        :raises MaestralApiError: if file could not be read for any other reason.
+        """
+
+        def str_from_date(d: datetime) -> str:
+            """Convert 'client_modified' metadata to string in local timezone"""
+            tz_date = d.replace(tzinfo=timezone.utc).astimezone()
+            return tz_date.strftime("%d %b %Y at %H:%M")
+
+        def download_rev(rev: str) -> Tuple[List[str], FileMetadata]:
+            """
+            Download a rev to a tmp file, read it and return the content + metadata
+            """
+
+            with tempfile.NamedTemporaryFile(mode="w+") as f:
+                md = self.client.download(dbx_path, f.name, rev=rev)
+
+                # Read from the file
+                try:
+                    with convert_api_errors(dbx_path=dbx_path, local_path=f.name):
+                        content = f.readlines()
+                except UnicodeDecodeError:
+                    raise UnsupportedFileTypeForDiff(
+                        "Failed to decode the file",
+                        "Only UTF-8 plain text files are currently supported.",
+                    )
+
+            return content, md
+
+        md_new = self.client.get_metadata(f"rev:{new_rev}", include_deleted=True)
+        md_old = self.client.get_metadata(f"rev:{old_rev}", include_deleted=True)
+
+        if md_new is None or md_old is None:
+            missing_rev = new_rev if md_new is None else old_rev
+            raise NotFoundError(
+                f"Could not a file with revision {missing_rev}",
+                "Use 'list_revisions' to list past revisions of a file.",
+            )
+
+        dbx_path = self.sync.correct_case(md_old.path_display)
+        local_path = self.sync.to_local_path(md_old.path_display)
+
+        # Check if a diff is possible
+        # If mime is None, proceed because most files without
+        # an extension are just text files
+        mime, _ = mimetypes.guess_type(dbx_path)
+        if mime is not None and not mime.startswith("text/"):
+            raise UnsupportedFileTypeForDiff(
+                f"Bad file type: '{mime}'", "Only files of type 'text/*' are supported."
+            )
+
+        # If new_rev is None, the local file is used, even if it isn't synced
+        if new_rev is None:
+            new_rev = "local version"
+            try:
+                with convert_api_errors(dbx_path=dbx_path, local_path=local_path):
+                    mtime = time.localtime(osp.getmtime(local_path))
+                    date_str_new = time.strftime("%d %b %Y at %H:%M", mtime)
+
+                    with open(local_path) as f:
+                        content_new = f.readlines()
+
+            except UnicodeDecodeError:
+                raise UnsupportedFileTypeForDiff(
+                    "Failed to decode the file",
+                    "Only UTF-8 plain text files are currently supported.",
+                )
+        else:
+            content_new, md_new = download_rev(new_rev)
+            date_str_new = str_from_date(md_new.client_modified)
+
+        content_old, md_old = download_rev(old_rev)
+        date_str_old = str_from_date(md_old.client_modified)
+
+        return list(
+            difflib.unified_diff(
+                content_old,
+                content_new,
+                fromfile=f"{dbx_path} ({old_rev})",
+                tofile=f"{dbx_path} ({new_rev})",
+                fromfiledate=date_str_old,
+                tofiledate=date_str_new,
+            )
+        )
+
     def restore(self, dbx_path: str, rev: str) -> StoneType:
         """
         Restore an old revision of a file.
@@ -881,6 +931,8 @@ class Maestral:
 
         self._check_linked()
 
+        logger.info(f"Restoring '{dbx_path} to {rev}'")
+
         res = self.client.restore(dbx_path, rev)
         return dropbox_stone_to_dict(res)
 
@@ -922,36 +974,14 @@ class Maestral:
         self._check_linked()
         self._check_dropbox_dir()
 
-        if not self.running:
-            self.monitor.start()
-
-    def resume_sync(self) -> None:
-        """
-        Resumes syncing if paused.
-
-        :raises NotLinkedError: if no Dropbox account is linked.
-        :raises NoDropboxDirError: if local Dropbox folder is not set up.
-        """
-
-        self._check_linked()
-        self._check_dropbox_dir()
-
-        self.monitor.resume()
-
-    def pause_sync(self) -> None:
-        """
-        Pauses the syncing if running.
-        """
-        if not self.paused:
-            self.monitor.pause()
+        self.monitor.start()
 
     def stop_sync(self) -> None:
         """
         Stops all syncing threads if running. Call :meth:`start_sync` to restart
         syncing.
         """
-        if self.running:
-            self.monitor.stop()
+        self.monitor.stop()
 
     def reset_sync_state(self) -> None:
         """
@@ -1044,7 +1074,7 @@ class Maestral:
             pass
         else:
             event_cls = DirDeletedEvent if osp.isdir(local_path) else FileDeletedEvent
-            with self.monitor.fs_event_handler.ignore(event_cls(local_path)):
+            with self.monitor.sync.fs_events.ignore(event_cls(local_path)):
                 delete(local_path)
 
     def include_item(self, dbx_path: str) -> None:
@@ -1225,8 +1255,8 @@ class Maestral:
 
         # pause syncing
         resume = False
-        if self.syncing:
-            self.pause_sync()
+        if self.running:
+            self.stop_sync()
             resume = True
 
         # housekeeping
@@ -1241,7 +1271,88 @@ class Maestral:
 
         # resume syncing
         if resume:
-            self.resume_sync()
+            self.start_sync()
+
+    def create_shared_link(
+        self,
+        dbx_path: str,
+        visibility: str = "public",
+        password: Optional[str] = None,
+        expires: Optional[float] = None,
+    ) -> StoneType:
+        """
+        Creates a shared link for the given ``dbx_path``. Returns a dictionary with
+        information regarding the link, including the URL, access permissions, expiry
+        time, etc. The shared link will grant read / download access only. Note that
+        basic accounts do not support password protection or expiry times.
+
+        :param dbx_path: Path to item on Dropbox.
+        :param visibility: Requested visibility of the shared link. Must be "public",
+            "team_only" or "password". The actual visibility may be different, depending
+            on the team and folder settings. Inspect the "link_permissions" entry of the
+            returned dictionary.
+        :param password: An optional password required to access the link. Will be
+            ignored if the visibility is not "password".
+        :param expires: An optional expiry time for the link as POSIX timestamp.
+        :returns: Shared link information as dict. See
+            :class:`dropbox.sharing.SharedLinkMetadata` for keys and values.
+        :raises ValueError: if visibility is 'password' but no password is provided.
+        :raises DropboxAuthError: in case of an invalid access token.
+        :raises DropboxServerError: for internal Dropbox errors.
+        :raises ConnectionError: if the connection to Dropbox fails.
+        :raises NotLinkedError: if no Dropbox account is linked.
+        """
+
+        self._check_linked()
+
+        if visibility not in ("public", "team_only", "password"):
+            raise ValueError("Visibility must be 'public', 'team_only', or 'password'")
+
+        if visibility == "password" and not password:
+            raise ValueError("Please specify a password")
+
+        link_info = self.client.create_shared_link(
+            dbx_path=dbx_path,
+            visibility=RequestedVisibility(visibility),
+            password=password,
+            expires=datetime.utcfromtimestamp(expires) if expires else None,
+        )
+
+        return dropbox_stone_to_dict(link_info)
+
+    def revoke_shared_link(self, url: str) -> None:
+        """
+        Revokes the given shared link. Note that any other links to the same file or
+        folder will remain valid.
+
+        :param url: URL of shared link to revoke.
+        :raises DropboxAuthError: in case of an invalid access token.
+        :raises DropboxServerError: for internal Dropbox errors.
+        :raises ConnectionError: if the connection to Dropbox fails.
+        :raises NotLinkedError: if no Dropbox account is linked.
+        """
+
+        self._check_linked()
+        self.client.revoke_shared_link(url)
+
+    def list_shared_links(self, dbx_path: Optional[str] = None) -> List[StoneType]:
+        """
+        Returns a list of all shared links for the given Dropbox path. If no path is
+        given, return all shared links for the account, up to a maximum of 1,000 links.
+
+        :param dbx_path: Path to item on Dropbox.
+        :returns: List of shared link information as dictionaries. See
+            :class:`dropbox.sharing.SharedLinkMetadata` for keys and values.
+        :raises DropboxAuthError: in case of an invalid access token.
+        :raises DropboxServerError: for internal Dropbox errors.
+        :raises ConnectionError: if the connection to Dropbox fails.
+        :raises NotLinkedError: if no Dropbox account is linked.
+        """
+
+        self._check_linked()
+        res = self.client.list_shared_links(dbx_path)
+
+        return [dropbox_stone_to_dict(link) for link in res.links]
 
     # ==== utility methods for front ends ==============================================
 
@@ -1362,6 +1473,8 @@ class Maestral:
             self._update_from_pre_v1_2_0()
         elif Version(updated_from) < Version("1.2.1"):
             self._update_from_pre_v1_2_1()
+        elif Version(updated_from) < Version("1.3.2"):
+            self._update_from_pre_v1_3_2()
 
         self.set_state("app", "updated_scripts_completed", __version__)
 
@@ -1416,29 +1529,42 @@ class Maestral:
 
                         batch_op.drop_constraint(constraint_name=name, type_="unique")
 
+    def _update_from_pre_v1_3_2(self) -> None:
+
+        if self._conf.get("app", "keyring") == "keyring.backends.OS_X.Keyring":
+            logger.info("Migrating keyring after update from pre v1.3.2")
+            self._conf.set("app", "keyring", "keyring.backends.macOS.Keyring")
+
     # ==== period async jobs ===========================================================
 
     def _schedule_task(self, coro: Awaitable) -> None:
+        """Schedules a task in our asyncio loop."""
 
         task = self._loop.create_task(coro)
         self._tasks.add(task)
 
     async def _periodic_refresh_info(self) -> None:
+        """Periodically refresh the account information from Dropbox servers."""
 
         await asyncio.sleep(60 * 5)
 
         while True:
             # update account info
             if self.client.auth.loaded:
-                # only run if we have loaded the keyring, we don't
-                # want to trigger any keyring access from here
-                if self.client.linked:
+
+                # Only run if we have loaded the keyring, we don't
+                # want to trigger any keyring access from here.
+
+                try:
                     await self._loop.run_in_executor(self._pool, self.get_account_info)
                     await self._loop.run_in_executor(self._pool, self.get_profile_pic)
+                except (ConnectionError, MaestralApiError):
+                    pass
 
             await sleep_rand(60 * 45)
 
     async def _period_update_check(self) -> None:
+        """Periodically check for software updates."""
 
         await asyncio.sleep(60 * 3)
 
@@ -1451,15 +1577,22 @@ class Maestral:
             await sleep_rand(60 * 60)
 
     async def _period_reindexing(self) -> None:
+        """
+        Trigger periodic reindexing, determined by the 'reindex_interval' setting. Don't
+        reindex if we are running on battery power.
+        """
 
         while True:
 
             if self.monitor.running.is_set():
                 elapsed = time.time() - self.sync.last_reindex
+                ac_state = get_ac_state()
+
                 reindexing_due = elapsed > self.monitor.reindex_interval
                 is_idle = self.monitor.idle_time > 20 * 60
+                has_ac_power = ac_state in (ACState.Connected, ACState.Undetermined)
 
-                if reindexing_due and is_idle:
+                if reindexing_due and is_idle and has_ac_power:
                     self.monitor.rebuild_index()
 
             await sleep_rand(60 * 5)
